import { useEffect, useState } from "react";
import { useNavigate } from "react-router-dom";
import { RootPaper } from "@ess-ics/ce-ui-common";
import {
  Alert,
  Autocomplete,
  Button,
  CircularProgress,
  LinearProgress,
  Stack,
  TextField,
  Typography
} from "@mui/material";
import { RepositoryOptions } from "./RepositoryOptions";
import { WITHOUT_REPO } from "./RepositoryType";
import { RepositoryName } from "./RepositoryName";
import { useTypingTimer } from "../../common/SearchBoxFilter/TypingTimer";
import { useCustomSnackbar } from "../../common/snackbar";

import { getErrorMessage } from "../../common/Helper";
import {
  useCreateIocMutation,
  useLazyFetchIocByNameQuery,
  useLazyListProjectsQuery
} from "../../../store/deployApi";

export function CreateIOC() {
  const navigate = useNavigate();
  const showSnackBar = useCustomSnackbar();
  const [namingEntity, setNamingEntity] = useState({});
  const [gitProject, setGitProject] = useState({});
  const [nameQuery, onNameKeyUp] = useTypingTimer({ interval: 500 });
  const [repoQuery, onRepoKeyUp] = useTypingTimer({ interval: 500 });
  const [selectedRepoOption, setSelectedRepoOption] = useState(WITHOUT_REPO);
  const [repoName, setRepoName] = useState("");

  const [
    getAllowedGitProjects,
    { data: allowedGitProjects, isLoading: loadingAllowedGitProjects }
  ] = useLazyListProjectsQuery();

  const [createIoc, { data: ioc, isLoading, error }] = useCreateIocMutation();

  const [getNames, { data: names, isLoading: loadingNames }] =
    useLazyFetchIocByNameQuery();

  // create the ioc on form submit
  const onSubmit = (event) => {
    event.preventDefault();
    createIoc({
      createIoc: {
        gitProjectId: gitProject.id,
        namingUuid: namingEntity ? namingEntity.uuid : undefined,
        repository_name: repoName ? repoName : undefined
      }
    });
  };

  const handleSelectRepoOption = (option) => {
    setSelectedRepoOption(option);
    setGitProject({});
    setRepoName("");
  };

  // fetch new names when name query changes
  useEffect(() => {
    if (nameQuery) {
      getNames(nameQuery);
    }
  }, [nameQuery, getNames]);

  useEffect(() => {
    if (repoQuery && repoQuery.length > 2) {
      getAllowedGitProjects({ query: repoQuery });
    }
  }, [repoQuery, getAllowedGitProjects]);

  // navigate home once ioc created
  useEffect(() => {
    if (ioc) {
      showSnackBar(
        selectedRepoOption === WITHOUT_REPO
          ? "IOC created"
          : "IOC created with a repository",
        "success"
      );
      navigate(`/iocs/${ioc.id}?&tab=Management`);
    }
  }, [ioc, showSnackBar, selectedRepoOption, navigate]);

  return (
    <RootPaper
      sx={{ display: "flex", flexDirection: "column", alignItems: "center" }}
    >
      <Stack
        component="form"
        onSubmit={onSubmit}
        gap={2}
        aria-live="polite"
        width="600px"
      >
        <Typography variant="h2">Create new IOC</Typography>
        <RepositoryOptions
          selectedRepoOption={selectedRepoOption}
          onSelectRepoOption={handleSelectRepoOption}
        />
        <Autocomplete
          autoHighlight
          id="nameAutocomplete"
          value={namingEntity}
          options={nameQuery ? (names ?? []) : []}
          loading={loadingNames}
          clearOnBlur={false}
          getOptionLabel={(option) => option?.name ?? ""}
          renderInput={(params) => (
            <TextField
              {...params}
              label="IOC name"
              variant="outlined"
              required
              InputProps={{
                ...params.InputProps,
                endAdornment: (
                  <>
                    {loadingNames ? (
                      <CircularProgress
                        color="inherit"
                        size={20}
                      />
                    ) : null}
                    {params.InputProps.endAdornment}
                  </>
                )
              }}
              disabled={isLoading}
              helperText={namingEntity ? namingEntity.description : ""}
            />
          )}
          onChange={(_, value) => setNamingEntity(value)}
          onInputChange={(event) => event && onNameKeyUp(event.nativeEvent)}
          autoSelect
        />
        {selectedRepoOption === WITHOUT_REPO ? (
          <Autocomplete
            autoHighlight
            id="gitId"
            value={gitProject}
            options={repoQuery || gitProject ? (allowedGitProjects ?? []) : []}
            loading={loadingAllowedGitProjects}
            clearOnBlur={false}
            getOptionLabel={(option) => {
              return option?.url ?? "";
            }}
            onChange={(_, value) => setGitProject(value)}
            onInputChange={(event) => event && onRepoKeyUp(event.nativeEvent)}
            renderInput={(params) => (
              <TextField
                {...params}
                label="Git repository"
                variant="outlined"
                fullWidth
                required
                InputProps={{
                  ...params.InputProps,
                  endAdornment: (
                    <>
                      {loadingAllowedGitProjects ? (
                        <CircularProgress
                          color="inherit"
                          size={20}
                        />
                      ) : null}
                      {params.InputProps.endAdornment}
                    </>
                  )
                }}
                disabled={isLoading}
              />
            )}
            autoSelect
          />
        ) : (
          <RepositoryName
            repoName={repoName}
            onRepoNameChange={(name) => setRepoName(name)}
          />
        )}

        {error ? (
          <Alert severity="error">{getErrorMessage(error)}</Alert>
        ) : null}
        {isLoading ? (
          <LinearProgress
            aria-busy="true"
            aria-label="Creating IOC, please wait"
          />
        ) : null}
        <Stack
          direction="row"
          justifyContent="flex-end"
        >
          <Button
            onClick={() => navigate("/")}
            color="primary"
            disabled={isLoading}
          >
            Cancel
          </Button>
          <Button
            color="primary"
            variant="contained"
            type="submit"
            disabled={
              isLoading || !namingEntity || selectedRepoOption === WITHOUT_REPO
                ? !gitProject
                : !repoName
            }
          >
            Create
          </Button>
        </Stack>
      </Stack>
    </RootPaper>
  );
}
