import { useEffect, useState, useMemo } from "react";
import { Typography, Stack, Box } from "@mui/material";
import {
  KeyValueTable,
  SimpleAccordion,
  AlertBannerList,
  ExternalLink,
  InternalLink,
  formatDateAndTime,
  EmptyValue,
  Duration,
  SingleStateStepper,
  STEPPER_STATES,
  AccessControl
} from "@ess-ics/ce-ui-common";
import { ActionTypeIconText } from "./JobIcons";
import { JobDetailsTable } from "./JobDetailsTable";
import { DeploymentJobOutput } from "../deployments/DeploymentJobOutput";
import { AWXJobDetails, Status } from "../../api/DataTypes";
import { JobDetails, AwxJobDetails } from "../../store/deployApi";

interface JobDetailsProps {
  jobDetail: JobDetails;
  awxJob: AwxJobDetails;
}

const createAlert = (type: Status, message: string) => {
  return {
    type,
    message
  };
};

const getDeploymentDetails = (operation: JobDetails) => {
  const startTime = operation?.startTime ?? operation?.createdAt;

  return {
    user: (
      <InternalLink
        to={`/user/${operation.createdBy}`}
        label={`User profile, ${operation.createdBy}`}
      >
        {operation.createdBy}
      </InternalLink>
    ),
    "AWX job link": operation?.jobUrl ? (
      <ExternalLink
        href={operation.jobUrl}
        label="AWX job"
      >
        {operation.jobUrl}
      </ExternalLink>
    ) : (
      <EmptyValue />
    ),
    "created time": operation?.createdAt ? (
      formatDateAndTime(operation.createdAt)
    ) : (
      <EmptyValue />
    ),
    "AWX job start time": operation?.startTime ? (
      formatDateAndTime(operation.startTime)
    ) : (
      <EmptyValue />
    ),
    duration: operation?.finishedAt ? (
      <Duration
        createOrStartDate={startTime && new Date(startTime)}
        finishedAt={operation?.finishedAt && new Date(operation?.finishedAt)}
        textOnly
      />
    ) : (
      <EmptyValue />
    )
  };
};

export const JobsDetails = ({ jobDetail: operation }: JobDetailsProps) => {
  const [alerts, setAlerts] = useState<
    { type?: Status; message?: string; link?: string }[]
  >(operation.alerts ?? []);
  const [expanded, setExpanded] = useState(false);

  const awxJob = useMemo(
    () => new AWXJobDetails(operation, operation.action),
    [operation]
  );

  useEffect(() => {
    const operationAlerts = operation.alerts ? operation.alerts : [];
    setAlerts([
      createAlert(awxJob.severity(), awxJob.message()),
      ...operationAlerts
    ]);
  }, [operation, awxJob]);

  return (
    <Stack spacing={2}>
      <Stack>{<AlertBannerList alerts={alerts} />}</Stack>
      <Box sx={{ paddingLeft: "46px" }}>
        <ActionTypeIconText action={operation.action} />
      </Box>
      <JobDetailsTable operation={operation} />
      <KeyValueTable
        obj={getDeploymentDetails(operation)}
        variant="overline"
        sx={{ border: 0 }}
        valueOptions={{ headerName: "" }}
      />
      <AccessControl
        allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
        renderNoAccess={() => <></>}
      >
        <SimpleAccordion
          expanded={expanded}
          onChange={() => setExpanded((prev) => !prev)}
          summary={
            <Stack
              flexDirection="row"
              alignItems="center"
            >
              <Typography
                component="h2"
                variant="h3"
                sx={{ marginRight: "8px" }}
              >
                Job log stream
              </Typography>
              <SingleStateStepper
                showToolTip
                step={{
                  state: operation?.status || STEPPER_STATES.UNKNOWN,
                  label: awxJob.message()
                }}
              />
            </Stack>
          }
        >
          <DeploymentJobOutput
            job={operation}
            isExpanded={expanded}
          />
        </SimpleAccordion>
      </AccessControl>
    </Stack>
  );
};
