import { useEffect, useCallback, useState, useContext } from "react";
import { IconButton, Typography, LinearProgress } from "@mui/material";
import ArrowBackIcon from "@mui/icons-material/ArrowBack";
import {
  RootPaper,
  KeyValueTable,
  GlobalAppBarContext,
  InternalLink,
  formatDateAndTime,
  EmptyValue
} from "@ess-ics/ce-ui-common";
import { useParams, useNavigate } from "react-router-dom";
import { Alias } from "./Alias";
import { RecordBadge } from "../../components/records/RecordBadge";
import {
  applicationTitle,
  onFetchEntityError
} from "../../components/common/Helper";
import { NotFoundView } from "../../components/navigation/NotFoundView/NotFoundView";

import { useGetRecordQuery } from "../../store/deployApi";

export function RecordDetailsView() {
  const { name } = useParams();
  const decodedName = decodeURIComponent(name);
  const [error, setError] = useState(null);

  const {
    data: record,
    error: fetchError,
    isLoading: recordLoading
  } = useGetRecordQuery({
    name: decodedName
  });

  useEffect(() => {
    if (fetchError) {
      onFetchEntityError(fetchError?.message, fetchError?.status, setError);
    }
  }, [fetchError]);

  const navigate = useNavigate();

  const { setTitle } = useContext(GlobalAppBarContext);

  useEffect(() => {
    if (record) {
      setTitle(applicationTitle(`Record Details: ${record.name}`));
    }
  }, [setTitle, record]);

  const handleClick = () => {
    navigate(-1);
  };

  const getSubset = useCallback((record) => {
    let subset = {
      "Alias for": (
        <Alias aliases={record?.aliasFor ? [record?.aliasFor] : []} />
      ),
      Description: record?.description || <EmptyValue />,
      Version: record?.iocVersion || <EmptyValue />,
      "Record type": record.recordType || <EmptyValue />,
      Host: record.hostId ? (
        <Typography>
          <InternalLink
            to={`/hosts/${record.hostId}`}
            label={`Host details, ${record?.hostName}`}
          >
            {record?.hostName}
          </InternalLink>
        </Typography>
      ) : (
        record?.hostName
      ),
      "Alias to": <Alias aliases={record?.aliasTo} />
    };

    for (const [key, value] of Object.entries(record.properties)) {
      if (key.toLowerCase().includes("time")) {
        subset["Last updated"] = formatDateAndTime(value);
      } else if (key.toLowerCase().includes("recordtype")) {
        subset["Record type"] = value;
      } else if (key.toLowerCase().includes("recorddesc")) {
        subset.Description = value;
      } else if (key.toLowerCase() === "iocversion") {
        subset["IOC Revision"] = value;
      } else if (
        key.toLowerCase() !== "iocid" &&
        key.toLowerCase() !== "alias"
      ) {
        subset[key.replaceAll("_", " ")] = value;
      }
    }

    return subset;
  }, []);

  if (decodedName.includes(";") || error) {
    return <NotFoundView />;
  }

  if (recordLoading || !record) {
    return (
      <RootPaper>
        <LinearProgress color="primary" />
      </RootPaper>
    );
  }

  return (
    <RootPaper>
      <>
        <IconButton
          color="inherit"
          onClick={handleClick}
          size="large"
        >
          <ArrowBackIcon />
        </IconButton>
        {record && (
          <>
            <RecordBadge
              record={record}
              sx={{ mt: 2.5 }}
            />
            <KeyValueTable
              obj={getSubset(record)}
              variant="overline"
              sx={{ border: 0 }}
              valueOptions={{ headerName: "" }}
            />
          </>
        )}
      </>
    </RootPaper>
  );
}
